@@ -25,7 +25,7 @@
 const char *
 cast5_get_info( int algo, size_t *keylen,
 		   size_t *blocksize, size_t *contextsize,
-		   void (**setkey)( void *c, byte *key, unsigned keylen ),
+		   int	(**setkey)( void *c, byte *key, unsigned keylen ),
 		   void (**encrypt)( void *c, byte *outbuf, byte *inbuf ),
 		   void (**decrypt)( void *c, byte *outbuf, byte *inbuf )
 		 );
