@@ -27,13 +27,6 @@ void secure_random_alloc(void);
 int  quick_random_gen( int onoff );
 void randomize_buffer( byte *buffer, size_t length, int level );
 byte *get_random_bits( size_t nbits, int level, int secure );
-void add_randomness( const void *buffer, size_t length, int source );
-
-
-/*-- the next two functions are implemented by all the system
-     specific source files rand-xxxx.c --*/
-void random_poll(void);
-void fast_random_poll(void);
-
+void fast_random_poll( void );
 
 #endif /*G10_RANDOM_H*/
