@@ -23,9 +23,10 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <unistd.h>
 
-/* #define MAINTAINER_OPTIONS */
+#define MAINTAINER_OPTIONS
 
 #include "packet.h"
 #include "iobuf.h"
@@ -107,6 +108,7 @@ enum cmd_and_opt_values { aNull = 0,
     oKeyring,
     oSecretKeyring,
     oDefaultKey,
+    oTrustedKey,
     oOptions,
     oDebug,
     oDebugAll,
@@ -147,6 +149,7 @@ enum cmd_and_opt_values { aNull = 0,
     oS2KCipher,
     oCharset,
     oNotDashEscaped,
+    oLockOnce,
 aTest };
 
 
@@ -223,7 +226,7 @@ static ARGPARSE_OPTS opts[] = {
     { oKeyring, "keyring"   ,2, N_("add this keyring to the list of keyrings")},
     { oSecretKeyring, "secret-keyring" ,2, N_("add this secret keyring to the list")},
     { oDefaultKey, "default-key" ,2, N_("|NAME|use NAME as default secret key")},
-    { oCharset, "charset"   , 2, N_("|NAME| set terminal charset to NAME") },
+    { oCharset, "charset"   , 2, N_("|NAME|set terminal charset to NAME") },
     { oOptions, "options"   , 2, N_("read options from file")},
 
     { oDebug, "debug"     ,4|16, N_("set debugging flags")},
@@ -233,6 +236,7 @@ static ARGPARSE_OPTS opts[] = {
     { oCompletesNeeded, "completes-needed", 1, N_("(default is 1)")},
     { oMarginalsNeeded, "marginals-needed", 1, N_("(default is 3)")},
     { oMaxCertDepth,	"max-cert-depth", 1, "@" },
+    { oTrustedKey, "trusted-key", 2, N_("|KEYID|ulimately trust this key")},
     { oLoadExtension, "load-extension" ,2, N_("|FILE|load extension module FILE")},
     { oRFC1991, "rfc1991",   0, N_("emulate the mode described in RFC1991")},
     { oS2KMode, "s2k-mode",  1, N_("|N|use passphrase mode N")},
@@ -262,7 +266,6 @@ static ARGPARSE_OPTS opts[] = {
 
   /* hidden options */
   #ifdef IS_G10MAINT
-    { aTest, "test"      , 0, "@" },
     { aExportOwnerTrust, "list-ownertrust",0 , "@"},  /* alias */
     { aListTrustDB, "list-trustdb",0 , "@"},
     { aListTrustPath, "list-trust-path",0, "@"},
@@ -296,6 +299,7 @@ static ARGPARSE_OPTS opts[] = {
     { oSetFilename, "set-filename", 2, "@" },
     { oComment, "comment", 2, "@" },
     { oNotDashEscaped, "not-dash-escaped", 0, "@" },
+    { oLockOnce, "lock-once", 0, "@" },
 {0} };
 
 
@@ -652,7 +656,6 @@ main( int argc, char **argv )
 	#else
 	  #ifdef MAINTAINER_OPTIONS
 	    case aPrimegen: set_cmd( &cmd, aPrimegen); break;
-	    case aTest: set_cmd( &cmd, aTest); break;
 	    case aGenRandom: set_cmd( &cmd, aGenRandom); break;
 	  #endif
 	  case aPrintMD: set_cmd( &cmd, aPrintMD); break;
@@ -706,6 +709,7 @@ main( int argc, char **argv )
 	  case oMaxCertDepth: opt.max_cert_depth = pargs.r.ret_int; break;
 	  case oTrustDBName: trustdb_name = pargs.r.ret_str; break;
 	  case oDefaultKey: opt.def_secret_key = pargs.r.ret_str; break;
+	  case oTrustedKey: register_trusted_key( pargs.r.ret_str ); break;
 	  case oNoOptions: break; /* no-options */
 	  case oHomedir: opt.homedir = pargs.r.ret_str; break;
 	  case oNoBatch: opt.batch = 0; break;
@@ -770,6 +774,7 @@ main( int argc, char **argv )
 						    pargs.r.ret_str);
 	    break;
 	  case oNotDashEscaped: opt.not_dash_escaped = 1; break;
+	  case oLockOnce: opt.lock_once = 1; break;
 
 	  default : pargs.err = configfp? 1:2; break;
 	}
@@ -836,6 +841,13 @@ main( int argc, char **argv )
 	log_error(_("invalid S2K mode; must be 0, 1 or 3\n"));
     }
 
+    {	const char *p = strusage(13);
+	for( ; *p && (isdigit(*p) || *p=='.'); p++ )
+	    ;
+	if( *p )
+	    log_info("NOTE: This is a development version!\n");
+    }
+
     if( log_get_errorcount(0) )
 	g10_exit(2);
 
@@ -1148,15 +1160,20 @@ main( int argc, char **argv )
 	if( argc < 1 || argc > 2 )
 	    wrong_args("--gen-random level [hex]");
 	{
+	    int c;
 	    int level = atoi(*argv);
 	    for(;;) {
-		byte *p = get_random_bits( 8, level, 0);
-		if( argc == 1 ) {
+		byte *p;
+		if( argc == 2 ) {
+		    p = get_random_bits( 8, level, 0);
 		    printf("%02x", *p );
 		    fflush(stdout);
 		}
-		else
-		    putchar(c&0xff);
+		else {
+		    p = get_random_bits( 800, level, 0);
+		    for(c=0; c < 100; c++ )
+			putchar( p[c] );
+		}
 		m_free(p);
 	    }
 	}
@@ -1192,10 +1209,6 @@ main( int argc, char **argv )
 	}
 	break;
 
-     #ifdef MAINTAINER_OPTIONS
-      case aTest: do_test( argc? atoi(*argv): 1 ); break;
-      #endif /* MAINTAINER OPTIONS */
-
       case aListTrustDB:
 	if( !argc )
 	    list_trustdb(NULL);
@@ -1418,12 +1431,5 @@ print_mds( const char *fname, int algo )
 
 
 
-#ifdef MAINTAINER_OPTIONS
-static void
-do_test(int times)
-{
-    m_check(NULL);
-}
-#endif /* MAINTAINER OPTIONS */
 #endif /* IS_G10MAINT */
 
