@@ -56,7 +56,7 @@ overwrite_filep( const char *fname )
 	return 0;  /* do not overwrite */
 
     tty_printf(_("File '%s' exists. "), fname);
-    if( cpr_get_answer_is_yes(N_("openfile.overwrite.okay"),
+    if( cpr_get_answer_is_yes("openfile.overwrite.okay",
 			       _("Overwrite (y/N)? ")) )
 	return 1;
     return 0;
