@@ -33,6 +33,7 @@
 #include "filter.h"
 #include "options.h"
 #include "main.h"
+#include "i18n.h"
 
 static int mpi_print_mode = 0;
 static int list_mode = 0;
@@ -105,7 +106,7 @@ unknown_pubkey_warning( int algo )
     algo &= 0xff;
     if( !unknown_pubkey_algos[algo] ) {
 	if( opt.verbose )
-	    log_info("can't handle public key algorithm %d\n", algo );
+	    log_info(_("can't handle public key algorithm %d\n"), algo );
 	unknown_pubkey_algos[algo] = 1;
     }
 }
@@ -287,7 +288,7 @@ parse( IOBUF inp, PACKET *pkt, int reqtype, ulong *retpos,
     }
 
     if( do_skip || !pkttype || (reqtype && pkttype != reqtype) ) {
-	skip_packet(inp, pkttype, pktlen);
+	skip_rest(inp, pktlen);
 	*skip = 1;
 	rc = 0;
 	goto leave;
@@ -406,10 +407,14 @@ static void
 skip_packet( IOBUF inp, int pkttype, unsigned long pktlen )
 {
     if( list_mode ) {
-	printf(":unknown packet: type %2d, length %lu\n", pkttype, pktlen );
+	if( pkttype == PKT_MARKER )
+	    fputs(":marker packet:\n", stdout );
+	else
+	    printf(":unknown packet: type %2d, length %lu\n", pkttype, pktlen);
 	if( pkttype ) {
 	    int c, i=0 ;
-	    printf("dump:");
+	    if( pkttype != PKT_MARKER )
+		fputs("dump:", stdout );
 	    if( iobuf_in_block_mode(inp) ) {
 		while( (c=iobuf_get(inp)) != -1 )
 		    dump_hex_line(c, &i);
@@ -610,9 +615,10 @@ parse_pubkeyenc( IOBUF inp, int pkttype, unsigned long pktlen, PACKET *packet )
 
 static void
 dump_sig_subpkt( int hashed, int type, int critical,
-		 const char * buffer, size_t buflen, size_t length )
+		 const byte *buffer, size_t buflen, size_t length )
 {
     const char *p=NULL;
+    int i;
 
     printf("\t%s%ssubpkt %d len %u (", /*)*/
 	      critical ? "critical ":"",
@@ -630,7 +636,8 @@ dump_sig_subpkt( int hashed, int type, int critical,
 	break;
       case SIGSUBPKT_SIG_EXPIRE:
 	if( length >= 4 )
-	    printf("sig expires %s", strtimestamp( buffer_to_u32(buffer) ) );
+	    printf("sig expires after %s",
+				     strtimevalue( buffer_to_u32(buffer) ) );
 	break;
       case SIGSUBPKT_EXPORTABLE:
 	if( length )
@@ -647,13 +654,16 @@ dump_sig_subpkt( int hashed, int type, int critical,
 	break;
       case SIGSUBPKT_KEY_EXPIRE:
 	if( length >= 4 )
-	    printf("key expires %s", strtimestamp( buffer_to_u32(buffer) ) );
+	    printf("key expires after %s",
+				    strtimevalue( buffer_to_u32(buffer) ) );
 	break;
       case SIGSUBPKT_ARR:
 	p = "additional recipient request";
 	break;
       case SIGSUBPKT_PREF_SYM:
-	p = "preferred symmetric algorithms";
+	fputs("pref-sym-algos:", stdout );
+	for( i=0; i < length; i++ )
+	    printf(" %d", buffer[i] );
 	break;
       case SIGSUBPKT_REV_KEY:
 	p = "revocation key";
@@ -668,10 +678,14 @@ dump_sig_subpkt( int hashed, int type, int critical,
 	p = "notation data";
 	break;
       case SIGSUBPKT_PREF_HASH:
-	p = "preferred hash algorithms";
+	fputs("pref-hash-algos:", stdout );
+	for( i=0; i < length; i++ )
+	    printf(" %d", buffer[i] );
 	break;
       case SIGSUBPKT_PREF_COMPR:
-	p = "preferred compression algorithms";
+	fputs("pref-zip-algos:", stdout );
+	for( i=0; i < length; i++ )
+	    printf(" %d", buffer[i] );
 	break;
       case SIGSUBPKT_KS_FLAGS:
 	p = "key server preferences";
